##Sep 27, 2021 Howard Chuang & Rogelio Oliva
#This is an online resource supplmenting Appendix A 
set.seed(5566)
#Below processes 1855 left-censored corrections
#import PCTIRI.rds file
#tleft is the raw data vector
tleft=readRDS("tleft.rds")
summary(tleft)


#Step (1) in Appendix A
tab=as.data.frame(table(tleft))
tab=cbind(tab,cumsum(tab$Freq))
#Step (2) in Appendix A
colnames(tab)[3]="cumFreq"
tab=cbind(tab,round(tab$cumFreq/sum(tab$Freq),4))
colnames(tab)[4]="LCDF"

#Create inputs for MDE
x.val=as.vector(tab$tleft)
x.val=as.numeric(x.val)
y.val=as.vector(tab$LCDF)

#Step (3) in Appendix A
#Define a L2 Loss function for exponential distribution (Exp)
expMDEL2=function(par){
  loss=rep(0,length(x.val))
  for(i in 1:length(x.val)){
    loss[i]=(pexp(x.val[i],exp(par[1]))-y.val[i])^2
  }
  sum(loss)
}

#Step (4) in Appendix A 
#Perform MDE for Exp
expfit_L2a=optim(-5,expMDEL2,method="Brent",lower=-6,upper=-2,
                 control=list(trace=1,abstol=1e-20,reltol=1e-12))
#Search for lambda in [exp(-6),exp(-2)], mapping to reasonable theta in [7,403]
#The abosolute and relative convergence tolerance 1e-20 & 1e-12 is more rigid 
#than the default setting.
expfit_L2a$par
exp(expfit_L2a$par)


#Step (3) in Appendix A
#Define a L2 Loss function for Weibull distribution (Weib)
weibMDEL2=function(par){
  loss=rep(0,length(x.val))
  for(i in 1:length(x.val)){
    loss[i]=(pweibull(x.val[i],exp(par[1]),exp(par[2]))
             -y.val[i])^2
  }
  sum(loss)
}

#Step (4) in Appendix A
#Perform MDE for Weib
weibfit_L2a=optim(c(-5,-5),weibMDEL2,hessian=FALSE,
                  control=list(trace=1,abstol=1e-20,reltol=1e-12))
#Search for two Weibull parameters without any constraints
#The abosolute and relative convergence tolerance 1e-20 & 1e-12 is more rigid 
#than the default setting.
weibfit_L2a$par
a.est=exp(weibfit_L2a$par[1])
b.est=exp(weibfit_L2a$par[2])


#Compute KS (Kolmogorov?VSmirnov) statistics, smaller the better
#Exp
max(y.val-pexp(x.val,exp(expfit_L2a$par)))
#Weib
max(y.val-pweibull(x.val,a.est,b.est))


#Visualize distribution-fitting
x11(width=8,height=5)
par(mar=c(4.5,4,3,2))
hist(tleft,freq=FALSE,main="Sec4: Prob Density",
     xlab='left-censored PCT-IRI')
lines(x.val,dexp(x.val,exp(expfit_L2a$par)),col='red',
      lty=3,lwd=2)
lines(x.val,dweibull(x.val,a.est,b.est),col='blue',
      lty=5,lwd=2)
#
legend(150,0.008,c("Exp","Weib"),col=c("red","blue"),
       lty=c(3,5),lwd=c(2,2),bty = "n",cex=1.2)


#Step (5) in Appendix A
##Derive bootstrapped standard errors (SE) & confidence intervals (CI)
set.seed(9527)
#Number of bootstrap replications
S=100 
#Exponential distribution
exp.boot.est=rep(0,S)
#Weibull distribution
weib.boot.est=matrix(0,ncol=2,nrow=S)
colnames(weib.boot.est)=c("a","b")

#Bootstrapped sampling 
for(s in 1:S){
  tleft.s=sample(tleft,size=length(tleft),replace=TRUE)
  tab.s=as.data.frame(table(tleft.s))
  tab.s=cbind(tab.s,cumsum(tab.s$Freq))
  colnames(tab.s)[3]="cumFreq"
  #Compute the empirical "left-censored CDF"
  tab.s=cbind(tab.s,round(tab.s$cumFreq/sum(tab.s$Freq),4))
  colnames(tab.s)[4]="LCDF"
  x.val.s=as.vector(tab.s$tleft.s)
  x.val.s=as.numeric(x.val.s)
  y.val.s=as.vector(tab.s$LCDF)
  #
  ##Exponential Fitting L2 Loss
  expMDEL2.s=function(par){
    loss.s=rep(0,length(x.val.s))
    for(i in 1:length(x.val.s)){
      loss.s[i]=(pexp(x.val.s[i],exp(par[1]))-y.val.s[i])^2
    }
    sum(loss.s)
  }
  #
  expfit_L2.s=optimize(expMDEL2.s,-5,lower=-6,upper=-2)
  exp.boot.est[s]=exp(expfit_L2.s$minimum)
  #
  ##Weibull
  weibMDEL2.s=function(par){
    loss.s=rep(0,length(x.val.s))
    for(i in 1:length(x.val.s)){
      loss.s[i]=(pweibull(x.val.s[i],exp(par[1]),exp(par[2]))
                 -y.val.s[i])^2
    }
    sum(loss.s)
  }
  #
  weibfit_L2.s=optim(c(-5,-5),weibMDEL2.s,hessian=FALSE,
                     control=list(trace=0,abstol=1e-20,
                                  reltol=1e-12))
  a.est.s=exp(weibfit_L2.s$par[1])
  b.est.s=exp(weibfit_L2.s$par[2])
  weib.boot.est[s,1]=a.est.s
  weib.boot.est[s,2]=b.est.s
  #
  if(s%%10==0){print(s)}
}

#Exponential lambda SE & 95%CI
sd(exp.boot.est)
quantile(exp.boot.est,c(0.025,0.975))

#Weibull a & b SEs & 95%CI
sd(weib.boot.est[,1])
sd(weib.boot.est[,2])
quantile(weib.boot.est[,1],c(0.025,0.975))
quantile(weib.boot.est[,2],c(0.025,0.975))
